﻿-- ------------------------------------------------------------------------------------- --
-- 					TradeSkillMaster_Crafting - AddOn by Sapu94							 	  	  --
--   http://wow.curse.com/downloads/wow-addons/details/tradeskillmaster_crafting.aspx    --
--																													  --
--		This addon is licensed under the CC BY-NC-ND 3.0 license as described at the		  --
--				following url: http://creativecommons.org/licenses/by-nc-nd/3.0/			 	  --
-- 	Please contact the author via email at sapu94@gmail.com with any questions or		  --
--		concerns regarding this license.																	  --
-- ------------------------------------------------------------------------------------- --

-- The following functions are contained attached to this file:
-- Enchanting:HasProfession() - determines if the player is an enchanter

-- The following "global" (within the addon) variables are initialized in this file:
-- Enchanting.vellumID - itemID for Enchanting Vellum
-- Enchanting.slot - hardcoded list of the slot of every craft

-- ===================================================================================== --


-- load the parent file (TSM) into a local variable and register this file as a module
local TSM = select(2, ...)
local L = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Crafting") -- loads the localization table
local Enchanting = TSM:NewModule("Enchanting", "AceEvent-3.0")

local debug = function(...) TSM:Debug(...) end -- for debugging

-- determines if the player is an enchanter
function Enchanting:HasProfession()
	local professionIDs = {7411, 7412, 7413, 13920, 28029, 51313, 74258, 110400}
	for _, id in pairs(professionIDs) do
		if IsSpellKnown(id) then return true end
	end
end

function Enchanting:GetSlot(itemID)
	if not itemID then return end
	return Enchanting.slot[itemID] or #Enchanting.slotList
end

Enchanting.vellumID = 38682

Enchanting.slotList = {L["2H Weapon"], L["Boots"], L["Bracers"], L["Chest"], L["Cloak"], L["Gloves"], L["Shield"], L["Staff"], L["Weapon"], L["Other"]}

-- looks up the itemID of the scroll that the enchant makes
-- index = spellID of the enchant
-- value = itemID of scroll
Enchanting.itemID = {
	[110764] = 79061, -- Enchant Weapon - Pandamonium
	[104338] = 74700, -- Enchant Bracer - Mastery
	[104385] = 74701, -- Enchant Bracer - Major Dodge
	[104389] = 74703, -- Enchant Bracer - Super Intellect
	[104390] = 74704, -- Enchant Bracer - Exceptional Strength
	[104391] = 74705, -- Enchant Bracer - Greater Agility
	[104392] = 74706, -- Enchant Chest - Super Resilience
	[104393] = 74707, -- Enchant Chest - Mighty Spirit
	[104395] = 74708, -- Enchant Chest - Glorious Stats
	[104397] = 74709, -- Enchant Chest - Superior Stamina
	[104398] = 74710, -- Enchant Cloak - Accuracy
	[104401] = 74711, -- Enchant Cloak - Greater Protection
	[104403] = 74712, -- Enchant Cloak - Superior Intellect
	[104404] = 74713, -- Enchant Cloak - Superior Critical Strike
	[104407] = 74715, -- Enchant Boots - Greater Haste
	[104408] = 74716, -- Enchant Boots - Greater Precision
	[104409] = 74717, -- Enchant Boots - Blurred Speed
	[104414] = 74718, -- Enchant Boots - Pandaren's Step
	[104416] = 74719, -- Enchant Gloves - Greater Haste
	[104417] = 74720, -- Enchant Gloves - Superior Expertise
	[104419] = 74721, -- Enchant Gloves - Super Strength
	[104420] = 74722, -- Enchant Gloves - Superior Mastery
	[104445] = 74729, -- Enchant Off-Hand - Major Intellect
	[130758] = 89737, -- Enchant Shield - Greater Parry
	[104425] = 74723, -- Enchant Weapon - Windsong
	[104427] = 74724, -- Enchant Weapon - Jade Spirit
	[104430] = 74725, -- Enchant Weapon - Elemental Force
	[104434] = 74726, -- Enchant Weapon - Dancing Steel
	[104440] = 74727, -- Enchant Weapon - Colossus
	[104442] = 74728, -- Enchant Weapon - River's Song
	[74132] = 52687, -- Enchant Gloves - Mastery
	[74189] = 52743, -- Enchant Boots - Earthen Vitality
	[74191] = 52744, -- Enchant Chest - Mighty Stats
	[74192] = 52745, -- Enchant Cloak - Lesser Power
	[74193] = 52746, -- Enchant Bracer - Speed
	[74197] = 52748, -- Enchant Weapon - Avalanche
	[74198] = 52749, -- Enchant Gloves - Haste
	[74199] = 52750, -- Enchant Boots - Haste
	[74200] = 52751, -- Enchant Chest - Stamina
	[74201] = 52752, -- Enchant Bracer - Critical Strike
	[74202] = 52753, -- Enchant Cloak - Intellect
	[74207] = 52754, -- Enchant Shield - Protection
	[74211] = 52755, -- Enchant Weapon - Elemental Slayer
	[74212] = 52756, -- Enchant Gloves - Exceptional Strength
	[74213] = 52757, -- Enchant Boots - Major Agility
	[74214] = 52758, -- Enchant Chest - Mighty Resilience
	[74220] = 52759, -- Enchant Gloves - Greater Expertise
	[74223] = 52760, -- Enchant Weapon - Hurricane
	[74225] = 52761, -- Enchant Weapon - Heartsong
	[74226] = 52762, -- Enchant Shield - Mastery
	[74229] = 52763, -- Enchant Bracer - Superior Dodge
	[74230] = 52764, -- Enchant Cloak - Critical Strike
	[74231] = 52765, -- Enchant Chest - Exceptional Spirit
	[74232] = 52766, -- Enchant Bracer - Precision
	[74234] = 52767, -- Enchant Cloak - Protection
	[74235] = 52768, -- Enchant Off-Hand - Superior Intellect
	[74236] = 52769, -- Enchant Boots - Precision
	[74237] = 52770, -- Enchant Bracer - Exceptional Spirit
	[74238] = 52771, -- Enchant Boots - Mastery
	[74239] = 52772, -- Enchant Bracer - Greater Expertise
	[74240] = 52773, -- Enchant Cloak - Greater Intellect
	[74242] = 52774, -- Enchant Weapon - Power Torrent
	[74244] = 52775, -- Enchant Weapon - Windwalk
	[74246] = 52776, -- Enchant Weapon - Landslide
	[74247] = 52777, -- Enchant Cloak - Greater Critical Strike
	[74248] = 52778, -- Enchant Bracer - Greater Critical Strike
	[74250] = 52779, -- Enchant Chest - Peerless Stats
	[74251] = 52780, -- Enchant Chest - Greater Stamina
	[74252] = 52781, -- Enchant Boots - Assassin's Step
	[74253] = 52782, -- Enchant Boots - Lavawalker
	[74254] = 52783, -- Enchant Gloves - Mighty Strength
	[74255] = 52784, -- Enchant Gloves - Greater Mastery
	[74256] = 52785, -- Enchant Bracer - Greater Speed
	[95471] = 68134, -- Enchant 2H Weapon - Mighty Agility
	[74195] = 52747, -- Enchant Weapon - Mending
	[96264] = 68784, -- Enchant Bracer - Agility
	[96261] = 68785, -- Enchant Bracer - Major Strength
	[96262] = 68786, -- Enchant Bracer - Mighty Intellect
	[27958] = 38912, -- Enchant Chest - Exceptional Mana
	[44484] = 38951, -- Enchant Gloves - Expertise
	[44488] = 38953, -- Enchant Gloves - Precision
	[44489] = 38954, -- Enchant Shield - Dodge
	[44492] = 38955, -- Enchant Chest - Mighty Health
	[44500] = 38959, -- Enchant Cloak - Superior Agility
	[44508] = 38961, -- Enchant Boots - Greater Spirit
	[44509] = 38962, -- Enchant Chest - Greater Mana Restoration
	[44513] = 38964, -- Enchant Gloves - Greater Assault
	[44528] = 38966, -- Enchant Boots - Greater Fortitude
	[44529] = 38967, -- Enchant Gloves - Major Agility
	[44555] = 38968, -- Enchant Bracer - Exceptional Intellect
	[60616] = 38971, -- Enchant Bracer - Assault
	[44582] = 38973, -- Enchant Cloak - Minor Power
	[44584] = 38974, -- Enchant Boots - Greater Vitality
	[44588] = 38975, -- Enchant Chest - Exceptional Resilience
	[44589] = 38976, -- Enchant Boots - Superior Agility
	[44591] = 38978, -- Enchant Cloak - Superior Dodge
	[44592] = 38979, -- Enchant Gloves - Exceptional Spellpower
	[44593] = 38980, -- Enchant Bracer - Major Spirit
	[44598] = 38984, -- Enchant Bracer - Expertise
	[60623] = 38986, -- Enchant Boots - Icewalker
	[44616] = 38987, -- Enchant Bracer - Greater Stats
	[44623] = 38989, -- Enchant Chest - Super Stats
	[44625] = 38990, -- Enchant Gloves - Armsman
	[44631] = 38993, -- Enchant Cloak - Shadow Armor
	[44635] = 38997, -- Enchant Bracer - Greater Spellpower
	[47672] = 39001, -- Enchant Cloak - Mighty Stamina
	[47766] = 39002, -- Enchant Chest - Greater Dodge
	[47898] = 39003, -- Enchant Cloak - Greater Speed
	[47899] = 39004, -- Enchant Cloak - Wisdom
	[47900] = 39005, -- Enchant Chest - Super Health
	[47901] = 39006, -- Enchant Boots - Tuskarr's Vitality
	[60606] = 44449, -- Enchant Boots - Assault
	[60609] = 44456, -- Enchant Cloak - Speed
	[60663] = 44457, -- Enchant Cloak - Major Agility
	[60668] = 44458, -- Enchant Gloves - Crusher
	[60692] = 44465, -- Enchant Chest - Powerful Stats
	[60763] = 44469, -- Enchant Boots - Greater Assault
	[60767] = 44470, -- Enchant Bracer - Superior Spellpower
	[44575] = 44815, -- Enchant Bracer - Greater Assault
	[62256] = 44947, -- Enchant Bracer - Major Stamina
	[42974] = 38948, -- Enchant Weapon - Executioner
	[44510] = 38963, -- Enchant Weapon - Exceptional Spirit
	[44524] = 38965, -- Enchant Weapon - Icebreaker
	[44576] = 38972, -- Enchant Weapon - Lifeward
	[44595] = 38981, -- Enchant 2H Weapon - Scourgebane
	[44621] = 38988, -- Enchant Weapon - Giant Slayer
	[44629] = 38991, -- Enchant Weapon - Exceptional Spellpower
	[44630] = 38992, -- Enchant 2H Weapon - Greater Savagery
	[44633] = 38995, -- Enchant Weapon - Exceptional Agility
	[46578] = 38998, -- Enchant Weapon - Deathfrost
	[59625] = 43987, -- Enchant Weapon - Black Magic
	[60621] = 44453, -- Enchant Weapon - Greater Potency
	[60691] = 44463, -- Enchant 2H Weapon - Massacre
	[60707] = 44466, -- Enchant Weapon - Superior Potency
	[60714] = 44467, -- Enchant Weapon - Mighty Spellpower
	[59621] = 44493, -- Enchant Weapon - Berserking
	[59619] = 44497, -- Enchant Weapon - Accuracy
	[62948] = 45056, -- Enchant Staff - Greater Spellpower
	[62959] = 45060, -- Enchant Staff - Spellpower
	[27951] = 37603, -- Enchant Boots - Dexterity
	[25086] = 38895, -- Enchant Cloak - Dodge
	[27899] = 38897, -- Enchant Bracer - Brawn
	[27905] = 38898, -- Enchant Bracer - Stats
	[27906] = 38899, -- Enchant Bracer - Greater Dodge
	[27911] = 38900, -- Enchant Bracer - Superior Healing
	[27913] = 38901, -- Enchant Bracer - Restore Mana Prime
	[27914] = 38902, -- Enchant Bracer - Fortitude
	[27917] = 38903, -- Enchant Bracer - Spellpower
	[27944] = 38904, -- Enchant Shield - Lesser Dodge
	[27945] = 38905, -- Enchant Shield - Intellect
	[27946] = 38906, -- Enchant Shield - Parry
	[27948] = 38908, -- Enchant Boots - Vitality
	[27950] = 38909, -- Enchant Boots - Fortitude
	[27954] = 38910, -- Enchant Boots - Surefooted
	[27957] = 38911, -- Enchant Chest - Exceptional Health
	[27960] = 38913, -- Enchant Chest - Exceptional Stats
	[27961] = 38914, -- Enchant Cloak - Major Armor
	[33990] = 38928, -- Enchant Chest - Major Spirit
	[33991] = 38929, -- Enchant Chest - Restore Mana Prime
	[33992] = 38930, -- Enchant Chest - Major Resilience
	[33993] = 38931, -- Enchant Gloves - Blasting
	[33994] = 38932, -- Enchant Gloves - Precise Strikes
	[33995] = 38933, -- Enchant Gloves - Major Strength
	[33996] = 38934, -- Enchant Gloves - Assault
	[33997] = 38935, -- Enchant Gloves - Major Spellpower
	[33999] = 38936, -- Enchant Gloves - Major Healing
	[34001] = 38937, -- Enchant Bracer - Major Intellect
	[34002] = 38938, -- Enchant Bracer - Lesser Assault
	[34003] = 38939, -- Enchant Cloak - Spell Penetration
	[34004] = 38940, -- Enchant Cloak - Greater Agility
	[34007] = 38943, -- Enchant Boots - Cat's Swiftness
	[34008] = 38944, -- Enchant Boots - Boar's Speed
	[34009] = 38945, -- Enchant Shield - Major Stamina
	[44383] = 38949, -- Enchant Shield - Resilience
	[46594] = 38999, -- Enchant Chest - Dodge
	[47051] = 39000, -- Enchant Cloak - Greater Dodge
	[27967] = 38917, -- Enchant Weapon - Major Striking
	[27968] = 38918, -- Enchant Weapon - Major Intellect
	[27971] = 38919, -- Enchant 2H Weapon - Savagery
	[27972] = 38920, -- Enchant Weapon - Potency
	[27975] = 38921, -- Enchant Weapon - Major Spellpower
	[27977] = 38922, -- Enchant 2H Weapon - Major Agility
	[27981] = 38923, -- Enchant Weapon - Sunfire
	[27982] = 38924, -- Enchant Weapon - Soulfrost
	[27984] = 38925, -- Enchant Weapon - Mongoose
	[28003] = 38926, -- Enchant Weapon - Spellsurge
	[28004] = 38927, -- Enchant Weapon - Battlemaster
	[34010] = 38946, -- Enchant Weapon - Major Healing
	[42620] = 38947, -- Enchant Weapon - Greater Agility
	[7418] = 38679, -- Enchant Bracer - Minor Health
	[7420] = 38766, -- Enchant Chest - Minor Health
	[7426] = 38767, -- Enchant Chest - Minor Absorption
	[7428] = 38768, -- Enchant Bracer - Minor Dodge
	[7443] = 38769, -- Enchant Chest - Minor Mana
	[7457] = 38771, -- Enchant Bracer - Minor Stamina
	[7748] = 38773, -- Enchant Chest - Lesser Health
	[7766] = 38774, -- Enchant Bracer - Minor Spirit
	[7771] = 38775, -- Enchant Cloak - Minor Protection
	[7776] = 38776, -- Enchant Chest - Lesser Mana
	[7779] = 38777, -- Enchant Bracer - Minor Agility
	[7782] = 38778, -- Enchant Bracer - Minor Strength
	[7857] = 38782, -- Enchant Chest - Health
	[7859] = 38783, -- Enchant Bracer - Lesser Spirit
	[7863] = 38785, -- Enchant Boots - Minor Stamina
	[7867] = 38786, -- Enchant Boots - Minor Agility
	[13378] = 38787, -- Enchant Shield - Minor Stamina
	[13419] = 38789, -- Enchant Cloak - Minor Agility
	[13421] = 38790, -- Enchant Cloak - Lesser Protection
	[13464] = 38791, -- Enchant Shield - Lesser Protection
	[13485] = 38792, -- Enchant Shield - Lesser Spirit
	[13501] = 38793, -- Enchant Bracer - Lesser Stamina
	[13536] = 38797, -- Enchant Bracer - Lesser Strength
	[13538] = 38798, -- Enchant Chest - Lesser Absorption
	[13607] = 38799, -- Enchant Chest - Mana
	[13612] = 38800, -- Enchant Gloves - Mining
	[13617] = 38801, -- Enchant Gloves - Herbalism
	[13620] = 38802, -- Enchant Gloves - Fishing
	[13622] = 38803, -- Enchant Bracer - Lesser Intellect
	[13626] = 38804, -- Enchant Chest - Minor Stats
	[13631] = 38805, -- Enchant Shield - Lesser Stamina
	[13635] = 38806, -- Enchant Cloak - Defense
	[13637] = 38807, -- Enchant Boots - Lesser Agility
	[13640] = 38808, -- Enchant Chest - Greater Health
	[13642] = 38809, -- Enchant Bracer - Spirit
	[13644] = 38810, -- Enchant Boots - Lesser Stamina
	[13646] = 38811, -- Enchant Bracer - Lesser Dodge
	[13648] = 38812, -- Enchant Bracer - Stamina
	[13659] = 38816, -- Enchant Shield - Spirit
	[13661] = 38817, -- Enchant Bracer - Strength
	[13663] = 38818, -- Enchant Chest - Greater Mana
	[13687] = 38819, -- Enchant Boots - Lesser Spirit
	[13689] = 38820, -- Enchant Shield - Lesser Parry
	[13698] = 38823, -- Enchant Gloves - Skinning
	[13700] = 38824, -- Enchant Chest - Lesser Stats
	[13746] = 38825, -- Enchant Cloak - Greater Defense
	[13815] = 38827, -- Enchant Gloves - Agility
	[13817] = 38828, -- Enchant Shield - Stamina
	[13822] = 38829, -- Enchant Bracer - Intellect
	[13836] = 38830, -- Enchant Boots - Stamina
	[13841] = 38831, -- Enchant Gloves - Advanced Mining
	[13846] = 38832, -- Enchant Bracer - Greater Spirit
	[13858] = 38833, -- Enchant Chest - Superior Health
	[13868] = 38834, -- Enchant Gloves - Advanced Herbalism
	[13882] = 38835, -- Enchant Cloak - Lesser Agility
	[13887] = 38836, -- Enchant Gloves - Strength
	[13890] = 38837, -- Enchant Boots - Minor Speed
	[13905] = 38839, -- Enchant Shield - Greater Spirit
	[13917] = 38841, -- Enchant Chest - Superior Mana
	[13931] = 38842, -- Enchant Bracer - Dodge
	[13935] = 38844, -- Enchant Boots - Agility
	[13939] = 38846, -- Enchant Bracer - Greater Strength
	[13941] = 38847, -- Enchant Chest - Stats
	[13945] = 38849, -- Enchant Bracer - Greater Stamina
	[13947] = 38850, -- Enchant Gloves - Riding Skill
	[13948] = 38851, -- Enchant Gloves - Minor Haste
	[20008] = 38852, -- Enchant Bracer - Greater Intellect
	[20009] = 38853, -- Enchant Bracer - Superior Spirit
	[20010] = 38854, -- Enchant Bracer - Superior Strength
	[20011] = 38855, -- Enchant Bracer - Superior Stamina
	[20012] = 38856, -- Enchant Gloves - Greater Agility
	[20013] = 38857, -- Enchant Gloves - Greater Strength
	[20015] = 38859, -- Enchant Cloak - Superior Defense
	[20016] = 38860, -- Enchant Shield - Vitality
	[20017] = 38861, -- Enchant Shield - Greater Stamina
	[20020] = 38862, -- Enchant Boots - Greater Stamina
	[20023] = 38863, -- Enchant Boots - Greater Agility
	[20024] = 38864, -- Enchant Boots - Spirit
	[20025] = 38865, -- Enchant Chest - Greater Stats
	[20026] = 38866, -- Enchant Chest - Major Health
	[20028] = 38867, -- Enchant Chest - Major Mana
	[23801] = 38881, -- Enchant Bracer - Mana Regeneration
	[23802] = 38882, -- Enchant Bracer - Healing Power
	[25072] = 38885, -- Enchant Gloves - Threat
	[25073] = 38886, -- Enchant Gloves - Shadow Power
	[25074] = 38887, -- Enchant Gloves - Frost Power
	[25078] = 38888, -- Enchant Gloves - Fire Power
	[25079] = 38889, -- Enchant Gloves - Healing Power
	[25080] = 38890, -- Enchant Gloves - Superior Agility
	[25083] = 38893, -- Enchant Cloak - Stealth
	[25084] = 38894, -- Enchant Cloak - Subtlety
	[44506] = 38960, -- Enchant Gloves - Gatherer
	[63746] = 45628, -- Enchant Boots - Lesser Accuracy
	[71692] = 50816, -- Enchant Gloves - Angler
	[7745] = 38772, -- Enchant 2H Weapon - Minor Impact
	[7786] = 38779, -- Enchant Weapon - Minor Beastslayer
	[7788] = 38780, -- Enchant Weapon - Minor Striking
	[7793] = 38781, -- Enchant 2H Weapon - Lesser Intellect
	[13380] = 38788, -- Enchant 2H Weapon - Lesser Spirit
	[13503] = 38794, -- Enchant Weapon - Lesser Striking
	[13529] = 38796, -- Enchant 2H Weapon - Lesser Impact
	[13653] = 38813, -- Enchant Weapon - Lesser Beastslayer
	[13655] = 38814, -- Enchant Weapon - Lesser Elemental Slayer
	[13693] = 38821, -- Enchant Weapon - Striking
	[13695] = 38822, -- Enchant 2H Weapon - Impact
	[13898] = 38838, -- Enchant Weapon - Fiery Weapon
	[13915] = 38840, -- Enchant Weapon - Demonslaying
	[13937] = 38845, -- Enchant 2H Weapon - Greater Impact
	[13943] = 38848, -- Enchant Weapon - Greater Striking
	[20029] = 38868, -- Enchant Weapon - Icy Chill
	[20030] = 38869, -- Enchant 2H Weapon - Superior Impact
	[20031] = 38870, -- Enchant Weapon - Superior Striking
	[20032] = 38871, -- Enchant Weapon - Lifestealing
	[20033] = 38872, -- Enchant Weapon - Unholy Weapon
	[20034] = 38873, -- Enchant Weapon - Crusader
	[20035] = 38874, -- Enchant 2H Weapon - Major Spirit
	[20036] = 38875, -- Enchant 2H Weapon - Major Intellect
	[21931] = 38876, -- Enchant Weapon - Winter's Might
	[22749] = 38877, -- Enchant Weapon - Spellpower
	[22750] = 38878, -- Enchant Weapon - Healing Power
	[23799] = 38879, -- Enchant Weapon - Strength
	[23800] = 38880, -- Enchant Weapon - Agility
	[23803] = 38883, -- Enchant Weapon - Mighty Spirit
	[23804] = 38884, -- Enchant Weapon - Mighty Intellect
	[27837] = 38896, -- Enchant 2H Weapon - Agility
	[64441] = 46026, -- Enchant Weapon - Blade Ward
	[64579] = 46098, -- Enchant Weapon - Blood Draining
}

-- used to lookup the slot number of the enchant used by TradeSkillMaster_Crafting
-- index = itemID of enchant's scroll
-- value = slot number
Enchanting.slot = {
	[68134] = 1, -- 2H
	[38981] = 1, -- 2H
	[38992] = 1, -- 2H
	[44463] = 1, -- 2H
	[38919] = 1, -- 2H
	[38922] = 1, -- 2H
	[38772] = 1, -- 2H
	[38781] = 1, -- 2H
	[38788] = 1, -- 2H
	[38796] = 1, -- 2H
	[38822] = 1, -- 2H
	[38845] = 1, -- 2H
	[38869] = 1, -- 2H
	[38874] = 1, -- 2H
	[38875] = 1, -- 2H
	[38896] = 1, -- 2H
	[74715] = 2, -- Boots
	[74716] = 2, -- Boots
	[74717] = 2, -- Boots
	[74718] = 2, -- Boots
	[52743] = 2, -- Boots
	[52750] = 2, -- Boots
	[52757] = 2, -- Boots
	[52769] = 2, -- Boots
	[52771] = 2, -- Boots
	[52781] = 2, -- Boots
	[52782] = 2, -- Boots
	[38961] = 2, -- Boots
	[38966] = 2, -- Boots
	[38974] = 2, -- Boots
	[38976] = 2, -- Boots
	[38986] = 2, -- Boots
	[39006] = 2, -- Boots
	[44449] = 2, -- Boots
	[44469] = 2, -- Boots
	[37603] = 2, -- Boots
	[38908] = 2, -- Boots
	[38909] = 2, -- Boots
	[38910] = 2, -- Boots
	[38943] = 2, -- Boots
	[38944] = 2, -- Boots
	[38785] = 2, -- Boots
	[38786] = 2, -- Boots
	[38807] = 2, -- Boots
	[38810] = 2, -- Boots
	[38819] = 2, -- Boots
	[38830] = 2, -- Boots
	[38837] = 2, -- Boots
	[38844] = 2, -- Boots
	[38862] = 2, -- Boots
	[38863] = 2, -- Boots
	[38864] = 2, -- Boots
	[45628] = 2, -- Boots
	[74700] = 3, -- Bracers
	[74701] = 3, -- Bracers
	[74703] = 3, -- Bracers
	[74704] = 3, -- Bracers
	[74705] = 3, -- Bracers
	[52746] = 3, -- Bracers
	[52752] = 3, -- Bracers
	[52763] = 3, -- Bracers
	[52766] = 3, -- Bracers
	[52770] = 3, -- Bracers
	[52772] = 3, -- Bracers
	[52778] = 3, -- Bracers
	[52785] = 3, -- Bracers
	[68784] = 3, -- Bracers
	[68785] = 3, -- Bracers
	[68786] = 3, -- Bracers
	[38968] = 3, -- Bracers
	[38971] = 3, -- Bracers
	[38980] = 3, -- Bracers
	[38984] = 3, -- Bracers
	[38987] = 3, -- Bracers
	[38997] = 3, -- Bracers
	[44470] = 3, -- Bracers
	[44815] = 3, -- Bracers
	[44947] = 3, -- Bracers
	[38897] = 3, -- Bracers
	[38898] = 3, -- Bracers
	[38899] = 3, -- Bracers
	[38900] = 3, -- Bracers
	[38901] = 3, -- Bracers
	[38902] = 3, -- Bracers
	[38903] = 3, -- Bracers
	[38937] = 3, -- Bracers
	[38938] = 3, -- Bracers
	[38679] = 3, -- Bracers
	[38768] = 3, -- Bracers
	[38771] = 3, -- Bracers
	[38774] = 3, -- Bracers
	[38777] = 3, -- Bracers
	[38778] = 3, -- Bracers
	[38783] = 3, -- Bracers
	[38793] = 3, -- Bracers
	[38797] = 3, -- Bracers
	[38803] = 3, -- Bracers
	[38809] = 3, -- Bracers
	[38811] = 3, -- Bracers
	[38812] = 3, -- Bracers
	[38817] = 3, -- Bracers
	[38829] = 3, -- Bracers
	[38832] = 3, -- Bracers
	[38842] = 3, -- Bracers
	[38846] = 3, -- Bracers
	[38849] = 3, -- Bracers
	[38852] = 3, -- Bracers
	[38853] = 3, -- Bracers
	[38854] = 3, -- Bracers
	[38855] = 3, -- Bracers
	[38881] = 3, -- Bracers
	[38882] = 3, -- Bracers
	[74706] = 4, -- Chest
	[74707] = 4, -- Chest
	[74708] = 4, -- Chest
	[74709] = 4, -- Chest
	[52744] = 4, -- Chest
	[52751] = 4, -- Chest
	[52758] = 4, -- Chest
	[52765] = 4, -- Chest
	[52779] = 4, -- Chest
	[52780] = 4, -- Chest
	[38912] = 4, -- Chest
	[38955] = 4, -- Chest
	[38962] = 4, -- Chest
	[38975] = 4, -- Chest
	[38989] = 4, -- Chest
	[39002] = 4, -- Chest
	[39005] = 4, -- Chest
	[44465] = 4, -- Chest
	[38911] = 4, -- Chest
	[38913] = 4, -- Chest
	[38928] = 4, -- Chest
	[38929] = 4, -- Chest
	[38930] = 4, -- Chest
	[38999] = 4, -- Chest
	[38766] = 4, -- Chest
	[38767] = 4, -- Chest
	[38769] = 4, -- Chest
	[38773] = 4, -- Chest
	[38776] = 4, -- Chest
	[38782] = 4, -- Chest
	[38798] = 4, -- Chest
	[38799] = 4, -- Chest
	[38804] = 4, -- Chest
	[38808] = 4, -- Chest
	[38818] = 4, -- Chest
	[38824] = 4, -- Chest
	[38833] = 4, -- Chest
	[38841] = 4, -- Chest
	[38847] = 4, -- Chest
	[38865] = 4, -- Chest
	[38866] = 4, -- Chest
	[38867] = 4, -- Chest
	[74710] = 5, -- Cloak
	[74711] = 5, -- Cloak
	[74712] = 5, -- Cloak
	[74713] = 5, -- Cloak
	[52745] = 5, -- Cloak
	[52753] = 5, -- Cloak
	[52764] = 5, -- Cloak
	[52767] = 5, -- Cloak
	[52773] = 5, -- Cloak
	[52777] = 5, -- Cloak
	[38959] = 5, -- Cloak
	[38973] = 5, -- Cloak
	[38978] = 5, -- Cloak
	[38993] = 5, -- Cloak
	[39001] = 5, -- Cloak
	[39003] = 5, -- Cloak
	[39004] = 5, -- Cloak
	[44456] = 5, -- Cloak
	[44457] = 5, -- Cloak
	[38895] = 5, -- Cloak
	[38914] = 5, -- Cloak
	[38939] = 5, -- Cloak
	[38940] = 5, -- Cloak
	[39000] = 5, -- Cloak
	[38775] = 5, -- Cloak
	[38789] = 5, -- Cloak
	[38790] = 5, -- Cloak
	[38806] = 5, -- Cloak
	[38825] = 5, -- Cloak
	[38835] = 5, -- Cloak
	[38859] = 5, -- Cloak
	[38893] = 5, -- Cloak
	[38894] = 5, -- Cloak
	[74719] = 6, -- Gloves
	[74720] = 6, -- Gloves
	[74721] = 6, -- Gloves
	[74722] = 6, -- Gloves
	[52687] = 6, -- Gloves
	[52749] = 6, -- Gloves
	[52756] = 6, -- Gloves
	[52759] = 6, -- Gloves
	[52783] = 6, -- Gloves
	[52784] = 6, -- Gloves
	[38951] = 6, -- Gloves
	[38953] = 6, -- Gloves
	[38964] = 6, -- Gloves
	[38967] = 6, -- Gloves
	[38979] = 6, -- Gloves
	[38990] = 6, -- Gloves
	[44458] = 6, -- Gloves
	[38931] = 6, -- Gloves
	[38932] = 6, -- Gloves
	[38933] = 6, -- Gloves
	[38934] = 6, -- Gloves
	[38935] = 6, -- Gloves
	[38936] = 6, -- Gloves
	[38800] = 6, -- Gloves
	[38801] = 6, -- Gloves
	[38802] = 6, -- Gloves
	[38823] = 6, -- Gloves
	[38827] = 6, -- Gloves
	[38831] = 6, -- Gloves
	[38834] = 6, -- Gloves
	[38836] = 6, -- Gloves
	[38850] = 6, -- Gloves
	[38851] = 6, -- Gloves
	[38856] = 6, -- Gloves
	[38857] = 6, -- Gloves
	[38885] = 6, -- Gloves
	[38886] = 6, -- Gloves
	[38887] = 6, -- Gloves
	[38888] = 6, -- Gloves
	[38889] = 6, -- Gloves
	[38890] = 6, -- Gloves
	[38960] = 6, -- Gloves
	[50816] = 6, -- Gloves
	[74729] = 7, -- Shield
	[89737] = 7, -- Shield
	[52754] = 7, -- Shield
	[52762] = 7, -- Shield
	[52768] = 7, -- Shield
	[38954] = 7, -- Shield
	[38904] = 7, -- Shield
	[38905] = 7, -- Shield
	[38906] = 7, -- Shield
	[38945] = 7, -- Shield
	[38949] = 7, -- Shield
	[38787] = 7, -- Shield
	[38791] = 7, -- Shield
	[38792] = 7, -- Shield
	[38805] = 7, -- Shield
	[38816] = 7, -- Shield
	[38820] = 7, -- Shield
	[38828] = 7, -- Shield
	[38839] = 7, -- Shield
	[38860] = 7, -- Shield
	[38861] = 7, -- Shield
	[45056] = 8, -- Staff
	[45060] = 8, -- Staff
	[79061] = 9, -- Weapon
	[74723] = 9, -- Weapon
	[74724] = 9, -- Weapon
	[74725] = 9, -- Weapon
	[74726] = 9, -- Weapon
	[74727] = 9, -- Weapon
	[74728] = 9, -- Weapon
	[52748] = 9, -- Weapon
	[52755] = 9, -- Weapon
	[52760] = 9, -- Weapon
	[52761] = 9, -- Weapon
	[52774] = 9, -- Weapon
	[52775] = 9, -- Weapon
	[52776] = 9, -- Weapon
	[52747] = 9, -- Weapon
	[38948] = 9, -- Weapon
	[38963] = 9, -- Weapon
	[38965] = 9, -- Weapon
	[38972] = 9, -- Weapon
	[38988] = 9, -- Weapon
	[38991] = 9, -- Weapon
	[38995] = 9, -- Weapon
	[38998] = 9, -- Weapon
	[43987] = 9, -- Weapon
	[44453] = 9, -- Weapon
	[44466] = 9, -- Weapon
	[44467] = 9, -- Weapon
	[44493] = 9, -- Weapon
	[44497] = 9, -- Weapon
	[38917] = 9, -- Weapon
	[38918] = 9, -- Weapon
	[38920] = 9, -- Weapon
	[38921] = 9, -- Weapon
	[38923] = 9, -- Weapon
	[38924] = 9, -- Weapon
	[38925] = 9, -- Weapon
	[38926] = 9, -- Weapon
	[38927] = 9, -- Weapon
	[38946] = 9, -- Weapon
	[38947] = 9, -- Weapon
	[38779] = 9, -- Weapon
	[38780] = 9, -- Weapon
	[38794] = 9, -- Weapon
	[38813] = 9, -- Weapon
	[38814] = 9, -- Weapon
	[38821] = 9, -- Weapon
	[38838] = 9, -- Weapon
	[38840] = 9, -- Weapon
	[38848] = 9, -- Weapon
	[38868] = 9, -- Weapon
	[38870] = 9, -- Weapon
	[38871] = 9, -- Weapon
	[38872] = 9, -- Weapon
	[38873] = 9, -- Weapon
	[38876] = 9, -- Weapon
	[38877] = 9, -- Weapon
	[38878] = 9, -- Weapon
	[38879] = 9, -- Weapon
	[38880] = 9, -- Weapon
	[38883] = 9, -- Weapon
	[38884] = 9, -- Weapon
	[46026] = 9, -- Weapon
	[46098] = 9, -- Weapon
}